@@ -22,18 +22,20 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#include "../../config.h"
+
 #include "database.h"
 
 void
 registerDatabaseBuiltIns (void)
 {
-#ifdef ENABLE_MSQL
+#ifdef HAVE_LIBMSQL
     registerDatabaseMsqlBuiltIns();
 #endif
-#ifdef ENABLE_MYSQL
+#ifdef HAVE_LIBMYSQLCLIENT
     registerDatabaseMysqlBuiltIns();
 #endif
-#ifdef ENABLE_ADABAS
+#ifdef HAVE_LIBADABAS
     registerDatabaseAdabasBuiltIns();
 #endif
 }
