@@ -142,7 +142,8 @@ valueNewHash (void)
 
 value*
 valueNewLambda (int numParams, int minVarArgs, int maxVarArgs,
-		dynstring *paramNames, char metaChar, bytecode *code, environment *env)
+		dynstring *paramNames, char metaChar, bytecode *code, environment *env,
+		int evalParams)
 {
     value *theValue = allocValue();
     int i;
@@ -163,6 +164,29 @@ valueNewLambda (int numParams, int minVarArgs, int maxVarArgs,
     theValue->v.lambda.metaChar = metaChar;
     theValue->v.lambda.code = code;
     theValue->v.lambda.env = env;
+    theValue->v.lambda.evalParams = evalParams;
+
+    return theValue;
+}
+
+value*
+valueNewEnvironment (environment *env)
+{
+    value *theValue = allocValue();
+
+    theValue->type = VALUE_ENVIRONMENT;
+    theValue->v.env.env = env;
+
+    return theValue;
+}
+
+value*
+valueNewBytecode (bytecode *code)
+{
+    value *theValue = allocValue();
+
+    theValue->type = VALUE_BYTECODE;
+    theValue->v.bytecode.code = code;
 
     return theValue;
 }
@@ -204,6 +228,12 @@ valueNewScalarFromValue (value *theValue)
 	case VALUE_LAMBDA :
 	    return valueNewScalarFromCString("<lambda>");
 
+	case VALUE_ENVIRONMENT :
+	    return valueNewScalarFromCString("<environment>");
+
+	case VALUE_BYTECODE :
+	    return valueNewScalarFromCString("<bytecode>");
+
 	case VALUE_WHATSIT :
 	    return valueNewScalarFromCString("<whatsit>");
     }
@@ -223,6 +253,8 @@ valueCopy (value *theValue)
 	case VALUE_UNDEFINED :
 	case VALUE_INTERNAL :
 	case VALUE_BUILT_IN :
+	case VALUE_ENVIRONMENT :
+	case VALUE_BYTECODE :
 	case VALUE_WHATSIT :
 	    *theCopy = *theValue;
 	    return theCopy;
@@ -303,6 +335,8 @@ valueCopyDeep (value *theValue)
 	case VALUE_UNDEFINED :
 	case VALUE_INTERNAL :
 	case VALUE_BUILT_IN :
+	case VALUE_ENVIRONMENT :
+	case VALUE_BYTECODE :
 	case VALUE_WHATSIT :
 	    *theCopy = *theValue;
 	    return theCopy;
@@ -454,6 +488,12 @@ valueAreEqual (value *value1, value *value2)
 	case VALUE_LAMBDA :
 	    return value1->v.lambda.code == value2->v.lambda.code;
 
+	case VALUE_ENVIRONMENT :
+	    return value1->v.env.env == value2->v.env.env;
+
+	case VALUE_BYTECODE :
+	    return value1->v.bytecode.code == value2->v.bytecode.code;
+
 	case VALUE_WHATSIT :
 	    return value1->v.whatsit.data == value2->v.whatsit.data;
 
@@ -516,6 +556,17 @@ valueHashDefine (value *theValue, dynstring *key, value *defValue)
     hash_insert(theValue->v.hash.hash, key->data, defValue);
 }
 
+void
+valueHashDefineCString (value *theValue, const char *key, value *defValue)
+{
+    value *hashEntry;
+
+    assert(theValue->type == VALUE_HASH);
+
+    hashEntry = (value*)hash_lookup(theValue->v.hash.hash, key);
+    hash_insert(theValue->v.hash.hash, key, defValue);
+}
+
 value*
 valueHashLookup (value *theValue, dynstring *key)
 {
@@ -532,6 +583,14 @@ valueHashLookupCString (value *theValue, const char *key)
     return (value*)hash_lookup(theValue->v.hash.hash, key);
 }
 
+void
+valueHashDelete (value *theValue, dynstring *key)
+{
+    assert(theValue->type == VALUE_HASH);
+
+    hash_delete(theValue->v.hash.hash, key->data);
+}
+
 int
 valueListLength (value *theValue)
 {
@@ -659,6 +718,8 @@ cStringForValueType (int type)
 	    "list",
 	    "hash",
 	    "lambda",
+	    "environment",
+	    "bytecode",
 	    "whatsit"
 	};
 
